c
c  wrap MPI floats and integer definitions to
c  proper formats
c
#if defined IRIX_host
       include "mpif.h"
       integer, parameter :: impireal    = mpi_real8
       integer, parameter :: impiinteger = mpi_integer
#elif defined ALPHA_host
       include "mpif.h"
       integer, parameter :: impireal    = mpi_real
       integer, parameter :: impiinteger = mpi_integer
#elif defined T3E_host
#include "mpif.h"
       integer, parameter :: impireal    = mpi_real8
       integer, parameter :: impiinteger = mpi_integer8
#elif defined C90_host
       include "mpif.h"
       integer, parameter :: impireal    = mpi_real8
       integer, parameter :: impiinteger = mpi_integer8
#elif defined SX5_host
       include "mpif.h"
       integer, parameter :: impireal    = mpi_real8
       integer, parameter :: impiinteger = mpi_integer8
#elif defined SR8000_host
       include "mpif.h"
       integer, parameter :: impireal    = mpi_real8
       integer, parameter :: impiinteger = mpi_integer4
#elif defined LINUX_host
       include "mpif.h"
       integer, parameter :: impireal    = mpi_real
       integer, parameter :: impiinteger = mpi_integer
#elif defined LINUX_real8_host
       include "mpif.h"
       integer, parameter :: impireal    = mpi_real8
       integer, parameter :: impiinteger = mpi_integer
#elif defined AIX_host
       include "mpif.h"
       integer, parameter :: impireal    = mpi_real8
       integer, parameter :: impiinteger = mpi_integer
#else
       include "mpif.h"
       integer, parameter :: impireal    = mpi_real
       integer, parameter :: impiinteger = mpi_integer
#endif
